package com.ruoyi.zhushi.service.impl;

import com.ruoyi.zhushi.entity.AppDTO;
import com.ruoyi.zhushi.entity.DkCheckInRecord;
import com.ruoyi.zhushi.entity.DkCheckInRecordDTO;
import com.ruoyi.zhushi.mapper.DkAttendanceGroupMapper;
import com.ruoyi.zhushi.mapper.DkRecordMapper;
import com.ruoyi.zhushi.service.DkAppService;
import com.ruoyi.zhushi.util.GeoDistanceUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.Duration;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeParseException;
import java.util.List;

@Service
public class DkAppServiceImpl implements DkAppService {

    @Autowired
    private DkAttendanceGroupMapper dkAttendanceGroupMapper;

    @Autowired
    private DkRecordMapper dkRecordMapper;

    @Override
    public String checkIn(AppDTO appDTO) {
        AppDTO appDTO1 = dkAttendanceGroupMapper.queryAttendanceGroupBYUserId(appDTO.getUserId());
        // 打卡开始时间
        String workStartTime = appDTO1.getWorkStartTime();
        if(workStartTime == null){
            return "系统设置错误";
        }

        // 根据经纬度计算两点之间的距离
        double distanceKm = GeoDistanceUtil.getDistanceKm(appDTO.getLat(), appDTO.getLng(), appDTO1.getLat(), appDTO1.getLng());
        if (distanceKm > appDTO.getRadius()) {
            return "超出范围";
        }

        // 解析给定的时分秒（格式：HH:mm:ss 或 HH:mm）
        boolean result = canSignIn(workStartTime);
        DkCheckInRecord dkCheckInRecord = new DkCheckInRecord();
        if(!result){
            boolean b = allowCheckIn(workStartTime, appDTO.getLateRange());
            if(!b){
                dkCheckInRecord.setSysUserId(appDTO.getUserId());
                dkCheckInRecord.setSysUserName(appDTO.getUserName());
                dkCheckInRecord.setCheckInTime(LocalDateTime.now());
                dkCheckInRecord.setCheckInStatus("2");
                dkCheckInRecord.setCheckInType("startCheckIn");
                dkRecordMapper.insert(dkCheckInRecord);
                return "迟到打卡";
            }
            dkCheckInRecord.setSysUserId(appDTO.getUserId());
            dkCheckInRecord.setSysUserName(appDTO.getUserName());
            dkCheckInRecord.setCheckInTime(LocalDateTime.now());
            dkCheckInRecord.setCheckInStatus("1");
            dkCheckInRecord.setCheckInType("startCheckIn");
            dkRecordMapper.insert(dkCheckInRecord);
            return "打卡成功";
        }


        dkCheckInRecord.setSysUserId(appDTO.getUserId());
        dkCheckInRecord.setSysUserName(appDTO.getUserName());
        dkCheckInRecord.setCheckInTime(LocalDateTime.now());
        dkCheckInRecord.setCheckInStatus("1");
        dkCheckInRecord.setCheckInType("startCheckIn");
        dkRecordMapper.insert(dkCheckInRecord);
        return "打卡成功";
    }

    // 判断是否可以签到（给定时间是否在当前时间之前）
    public static boolean canSignIn(String timeStr) {
        // 解析给定的时分秒（格式：HH:mm:ss 或 HH:mm）
        DateTimeFormatter formatter = timeStr.contains(":") ?
            (timeStr.length() > 5 ? DateTimeFormatter.ofPattern("HH:mm:ss") :
                DateTimeFormatter.ofPattern("HH:mm")) :
            DateTimeFormatter.ofPattern("HHmmss");

        LocalTime givenTime = LocalTime.parse(timeStr, formatter);
        LocalTime currentTime = LocalTime.now();

        // 如果给定时间在当前时间之前，则可以签到
        return givenTime.isBefore(currentTime);
    }

    public static boolean allowCheckIn(String timeStr, long maxDifferenceMinutes) {
        try {
            // 解析输入的时间字符串
            DateTimeFormatter formatter = timeStr.contains(":") ?
                (timeStr.length() > 5 ? DateTimeFormatter.ofPattern("HH:mm:ss") :
                    DateTimeFormatter.ofPattern("HH:mm")) :
                DateTimeFormatter.ofPattern("HHmmss");

            LocalTime inputTime = LocalTime.parse(timeStr, formatter);
            LocalTime currentTime = LocalTime.now();

            // 计算时间差（绝对值）
            Duration duration = Duration.between(inputTime, currentTime);
            long diffMinutes = Math.abs(duration.toMinutes());

            // 判断是否超过最大允许时间差
            return diffMinutes <= maxDifferenceMinutes;
        } catch (DateTimeParseException e) {
            System.err.println("时间格式解析错误: " + e.getMessage());
            return false; // 格式错误时默认不允许打卡
        }
    }

    @Override
    public String checkOut(AppDTO appDTO) {
        AppDTO appDTO1 = dkAttendanceGroupMapper.queryAttendanceGroupBYUserId(appDTO.getUserId());
        // 根据经纬度计算两点之间的距离
        double distanceKm = GeoDistanceUtil.getDistanceKm(appDTO.getLat(), appDTO.getLng(), appDTO1.getLat(), appDTO1.getLng());
        if (distanceKm > appDTO.getRadius()){
            return "超出范围";
        }

        // 打卡开始时间
        String workEndTime = appDTO1.getWorkEndTime();
        if(workEndTime == null){
            return "系统设置错误";
        }

        // 解析给定的时分秒（格式：HH:mm:ss 或 HH:mm）
        boolean result = canSignIn(workEndTime);
        if(result){
            if(appDTO1.getAllowEarly()){
                boolean b = allowCheckIn(workEndTime, appDTO.getEarlyRange());
                if(!b){
                    return "不允许超出提前打卡范围";
                }
                // 签到成功
                DkCheckInRecord  dkCheckInRecord = new DkCheckInRecord();
                dkCheckInRecord.setSysUserId(appDTO.getUserId());
                dkCheckInRecord.setSysUserName(appDTO.getUserName());
                dkCheckInRecord.setCheckInTime(LocalDateTime.now());
                dkCheckInRecord.setCheckInStatus("0");
                dkCheckInRecord.setCheckInType("endCheckIn");
                dkRecordMapper.insertOrUpdate(dkCheckInRecord);
                return "打卡成功";
            }
            DkCheckInRecord  dkCheckInRecord = new DkCheckInRecord();
            dkCheckInRecord.setSysUserId(appDTO.getUserId());
            dkCheckInRecord.setSysUserName(appDTO.getUserName());
            dkCheckInRecord.setCheckInTime(LocalDateTime.now());
            dkCheckInRecord.setCheckInStatus("0");
            dkCheckInRecord.setCheckInType("endCheckIn");
            dkRecordMapper.insertOrUpdate(dkCheckInRecord);
            return "提前打卡";
        }else{
            DkCheckInRecord  dkCheckInRecord = new DkCheckInRecord();
            dkCheckInRecord.setSysUserId(appDTO.getUserId());
            dkCheckInRecord.setSysUserName(appDTO.getUserName());
            dkCheckInRecord.setCheckInTime(LocalDateTime.now());
            dkCheckInRecord.setCheckInStatus("0");
            dkCheckInRecord.setCheckInType("endCheckIn");
            dkRecordMapper.insertOrUpdate(dkCheckInRecord);
            return "打卡成功";
        }
    }

    @Override
    public List<DkCheckInRecord> getCurrentDayRecord(AppDTO appDTO) {
        List<DkCheckInRecord> currentDayRecords = dkRecordMapper.getCurrentDayRecord(appDTO.getUserId());
        return currentDayRecords;
    }
}
